Package: seriation
Type: Package
Title: Infrastructure for Ordering Objects Using Seriation
Version: 1.5.8
Date: 2025-08-20
Authors@R: c(
	person("Michael", "Hahsler", role = c("aut", "cre", "cph"),
	  email = "mhahsler@lyle.smu.edu",
	  comment = c(ORCID = "0000-0003-2716-1405")),
	person("Christian", "Buchta", role = c("aut", "cph")),
	person("Kurt", "Hornik", role = c("aut", "cph"),
	  comment = c(ORCID = "0000-0003-4198-9911")),
	person("David", "Barnett", role = c("ctb")),
	person("Michael", "Brusco", role = c("ctb", "cph")),
	person("Michael", "Friendly", role = c("ctb")),
	person("Hans-Friedrich", "Koehn", role = c("ctb", "cph")),
	person("Fionn", "Murtagh", role = c("ctb", "cph")),
	person("Stephanie", "Stahl", role = c("ctb", "cph")))
Description: Infrastructure for ordering objects with an implementation of several
    seriation/sequencing/ordination techniques to reorder matrices, dissimilarity
    matrices, and dendrograms. Also provides (optimally) reordered heatmaps,
    color images and clustering visualizations like dissimilarity plots, and
    visual assessment of cluster tendency plots (VAT and iVAT). Hahsler et al (2008) <doi:10.18637/jss.v025.i03>.
Classification/ACM: G.1.6, G.2.1, G.4
URL: https://github.com/mhahsler/seriation
BugReports: https://github.com/mhahsler/seriation/issues
Depends: R (>= 2.14.0)
Imports: ca, cluster, colorspace, foreach, gclus, grDevices, grid,
        MASS, qap, registry, stats, TSP, vegan
Suggests: dbscan, DendSer, dendextend, doParallel, GA, ggplot2, keras,
        Rtsne, scales, smacof, tensorflow, testthat, umap
Encoding: UTF-8
RoxygenNote: 7.3.2
License: GPL-3
Copyright: The code in src/bbwrcg.f, src/arsa.f and src/bburcg.f are
        Copyright (C) 2005 M. Brusco, H.F. Koehn, and S. Stahl. All
        other code is Copyright (C) Michael Hahsler, Christian Buchta,
        and Kurt Hornik.
Collate: 'AAA_check_installed.R' 'AAA_color_palette.R' 'AAA_defaults.R'
        'AAA_map.R' 'AAA_parameters.R' 'AAA_registry_criterion.R'
        'AAA_registry_seriate.R' 'AAA_seriation-package.R'
        'Chameleon.R' 'Irish.R' 'Munsingen.R' 'Psych24.R'
        'SupremeCourt.R' 'Townships.R' 'VAT.R' 'Wood.R' 'Zoo.R'
        'bertinplot.R' 'criterion.R' 'criterion.array.R'
        'criterion.dist.R' 'criterion.matrix.R' 'dissplot.R'
        'get_order.R' 'ggVAT.R' 'ggbertinplot.R' 'ggdissplot.R'
        'hmap.R' 'gghmap.R' 'pimage.R' 'ggpimage.R' 'grid_helpers.R'
        'lines_and_ordered_data.R' 'lle.R' 'permute.R'
        'register_DendSer.R' 'register_GA.R' 'register_optics.R'
        'register_smacof.R' 'register_tsne.R' 'register_umap.R'
        'reorder.hclust.R' 'robinson.R' 'ser_dist.R'
        'ser_permutation.R' 'ser_permutation_vector.R'
        'ser_permutation_vector2matrix.R' 'seriate.R' 'seriate.dist.R'
        'seriate.matrix.R' 'seriate.array.R' 'seriate.data.frame.R'
        'seriate.table.R' 'seriate_AOE.R' 'seriate_ARSA_Branch-Bound.R'
        'seriate_TSP.R' 'seriate_BEA.R' 'seriate_BK.R' 'seriate_CA.R'
        'seriate_GSA.R' 'seriate_HC.R' 'seriate_LLE.R' 'seriate_MDS.R'
        'seriate_Mean.R' 'seriate_PCA.R' 'seriate_QAP.R'
        'seriate_R2E.R' 'seriate_SGD.R' 'seriate_SPIN.R'
        'seriate_VAT.R' 'seriate_best.R' 'seriate_enumerate.R'
        'seriate_heatmap.R' 'seriate_identity.R' 'seriate_random.R'
        'seriate_reverse.R' 'seriate_spectral.R' 'seriate_vegan.R'
        'uniscale.R'
NeedsCompilation: yes
Packaged: 2025-08-20 17:01:36 UTC; mhahsler
Author: Michael Hahsler [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-2716-1405>),
  Christian Buchta [aut, cph],
  Kurt Hornik [aut, cph] (ORCID: <https://orcid.org/0000-0003-4198-9911>),
  David Barnett [ctb],
  Michael Brusco [ctb, cph],
  Michael Friendly [ctb],
  Hans-Friedrich Koehn [ctb, cph],
  Fionn Murtagh [ctb, cph],
  Stephanie Stahl [ctb, cph]
Maintainer: Michael Hahsler <mhahsler@lyle.smu.edu>
Repository: CRAN
Date/Publication: 2025-08-20 18:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 05:23:40 UTC; unix
Archs: seriation.so.dSYM
