# Generated by roxygen2: do not edit by hand

S3method("[",ser_permutation)
S3method(c,ser_permutation)
S3method(c,ser_permutation_vector)
S3method(criterion,array)
S3method(criterion,data.frame)
S3method(criterion,default)
S3method(criterion,dist)
S3method(criterion,matrix)
S3method(criterion,table)
S3method(get_order,default)
S3method(get_order,dendrogram)
S3method(get_order,hclust)
S3method(get_order,integer)
S3method(get_order,numeric)
S3method(get_order,ser_permutation)
S3method(get_order,ser_permutation_vector)
S3method(ggpimage,default)
S3method(ggpimage,dist)
S3method(ggpimage,matrix)
S3method(length,ser_permutation_vector)
S3method(permute,array)
S3method(permute,character)
S3method(permute,data.frame)
S3method(permute,default)
S3method(permute,dendrogram)
S3method(permute,dist)
S3method(permute,hclust)
S3method(permute,list)
S3method(permute,matrix)
S3method(permute,numeric)
S3method(permute,table)
S3method(pimage,data.frame)
S3method(pimage,default)
S3method(pimage,dist)
S3method(pimage,matrix)
S3method(pimage,table)
S3method(plot,reordered_cluster_dissimilarity_matrix)
S3method(print,criterion_method)
S3method(print,reordered_cluster_dissimilarity_matrix)
S3method(print,ser_permutation)
S3method(print,ser_permutation_vector)
S3method(print,seriation_method)
S3method(reorder,hclust)
S3method(rev,ser_permutation_vector)
S3method(seriate,array)
S3method(seriate,data.frame)
S3method(seriate,default)
S3method(seriate,dist)
S3method(seriate,matrix)
S3method(seriate,table)
S3method(summary,ser_permutation)
S3method(summary,ser_permutation_vector)
export(LS_insert)
export(LS_mixed)
export(LS_reverse)
export(LS_swap)
export(MDS_stress)
export(VAT)
export(bertin_cut_line)
export(bertinplot)
export(bluered)
export(blues)
export(create_lines_data)
export(create_ordered_data)
export(criterion)
export(dissplot)
export(gaperm_mixedMutation)
export(get_config)
export(get_criterion_method)
export(get_method)
export(get_order)
export(get_permutation_matrix)
export(get_rank)
export(get_seriation_method)
export(ggVAT)
export(ggbertinplot)
export(ggdissplot)
export(gghmap)
export(ggiVAT)
export(ggpimage)
export(grays)
export(greenred)
export(greens)
export(greys)
export(hmap)
export(iVAT)
export(is.robinson)
export(list_criterion_methods)
export(list_seriation_methods)
export(lle)
export(panel.bars)
export(panel.blocks)
export(panel.circles)
export(panel.lines)
export(panel.rectangles)
export(panel.squares)
export(panel.tiles)
export(path_dist)
export(permutation_matrix2vector)
export(permutation_vector2matrix)
export(permute)
export(pimage)
export(plot_config)
export(random.robinson)
export(reds)
export(register_DendSer)
export(register_GA)
export(register_optics)
export(register_smacof)
export(register_tsne)
export(register_umap)
export(registry_criterion)
export(registry_seriate)
export(ser_align)
export(ser_cor)
export(ser_dist)
export(ser_permutation)
export(ser_permutation_vector)
export(seriate)
export(seriate_best)
export(seriate_improve)
export(seriate_rep)
export(set_criterion_method)
export(set_seriation_method)
export(uniscale)
import("TSP")
import("grid")
import(registry)
importFrom(ca,ca)
importFrom(foreach,`%do%`)
importFrom(foreach,`%dopar%`)
importFrom(foreach,times)
importFrom(graphics,plot)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(stats,as.dist)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,order.dendrogram)
importFrom(stats,prcomp)
importFrom(stats,reorder)
importFrom(stats,rnorm)
importFrom(stats,runif)
useDynLib(seriation, .registration=TRUE)
