#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



## wrapper to greedy ordering inspired by F. Murtagh
## actually a hierarchical cluster algorithm.

## ceeboo 2005

hclust_greedy <- function(dist) {
    if (!inherits(dist, "dist"))
        stop("Argument 'dist' is not of class 'dist'.")
    if (!is.double(dist))
    mode(dist) <- "double"
    obj <- .Call("order_greedy", dist)
    names(obj) <- c("merge", "order", "height");
    class(obj) <- "hclust"
    obj
}

